# -*- coding=UTF-8 -*-
# Urban Dictionary Statics
# V1.2 (2015-07-15 17:10)
# 作者: firefly(dance.of.firefly@gmail.com)
#
# 作用: 统抓取情况（可写入准确的进度信息）
# Original from: http://www.jb51.net/article/47108.htm (Refined by firefly)
# 

import sys
import os
import time
from os import path
from multiprocessing import Pool


WL_folder = 'WordLists'     # 单词列表文件夹
MDX_folder = 'MDXs'         # 生成的MDX制作源文件
FAILED_folder = 'Failed'    # 在线抓取信息失败的文件夹：应对网络中断
PROGRESS_folder = 'Progress' # 处理进度的文件夹（当前处理到的行号）：应对断电
ABANDON_folder = 'Abandon'  # 存放由于所有解释和例句都不符合要求而舍弃掉的词语

def block(file,size=65536):
    while True:
        nb = file.read(size)
        if not nb:
           break
        yield nb

################################################################################
# 获取单个文本文件的行数
#
# 返回值：单个文本文件的行数
################################################################################
def getLineCount(filename):
    with open(filename,"r",encoding="utf-8") as f:
        return sum(line.count("\n") for line in block(f))

################################################################################
# 多进程获取当前目录的单个文件夹下所有文件的行数
#
# 返回值：flc_info: 文件夹下每个文件的详细行数（Key为文件名，Value为行数数值）
#         total: 所有文件总行数
################################################################################
def countLinesOfFolder(folder):
    folder_path = ''.join([os.getcwd(), path.sep, folder])
    files = [(folder_path + path.sep + f) for f in os.listdir(folder_path) if os.path.isfile(folder_path + path.sep + f) ]
    numOfProcs = len(files)  #处理分配的进程数
    if (numOfProcs!=0) :
        pool = Pool(numOfProcs)
        counts = pool.map(getLineCount, files)
    else :
        counts = []
    total = sum(c for c in counts)
    
    flc_info = {}
    for index in range(0, len(files)) :
        flc_info[os.path.basename(files[index])] = counts[index]
#    print(flc_info)
    return flc_info, total

################################################################################
# dump information
#
# tries to write success information into file
# Success info contains: time, success message
################################################################################
def dumpInfo(data):
    fname='statics.txt'
    fpath = ''.join([os.getcwd(), path.sep, fname])
    fw = open(fpath, 'w', encoding='utf-8')  # open or create file
    timestamp = time.asctime( time.localtime(time.time()) )
    data = timestamp +'\n' + data +'\n'
    try:
        fw.write(data)
    finally:
        fw.close()


if __name__ == "__main__":
    start_time = time.time()

    wl_info,wl_total = countLinesOfFolder(WL_folder)
    fl_info,fl_total = countLinesOfFolder(FAILED_folder)
    al_info,al_total = countLinesOfFolder(ABANDON_folder)
    # 处理 MDX 文件行数读取计算结果
    ml_info,ml_total = countLinesOfFolder(MDX_folder)
    prgs_info = {}
    for ml in ml_info :
        if (ml_info[ml] % 3 ==0) :
            ml_info[ml] //= 3
            #计算当前进度信息(当前刚处理过的行号)并写入文件
            prgs_info[ml] = ml_info.get(ml,0) + fl_info.get(ml,0) + al_info.get(ml,0)
            progress_fpath = ''.join([os.getcwd(), path.sep, PROGRESS_folder, path.sep, ml])
            progress_file = open(progress_fpath,'w', encoding='utf-8')  # open or create file in write mode
            progress_file.write(str(prgs_info[ml]))
            progress_file.close()
        else :
            ml_info[ml] /= 3
            print(ml +' in '+ MDX_folder +'存在问题，请手动检查！')
    ml_total /= 3
    prgs_total = sum(prgs_info[c] for c in prgs_info)
    
    # 打印统计结果
    result = '~~~~~~~~~~统计结果：~~~~~~~~~~\n'
    result += 'Wordlists     Total    Progress        (OK     Failed     Abandon)   Percent\n'
    for i in wl_info :
        percent = "%3.2f%%" % ( prgs_info.get(i,0) * 100.00 / wl_info.get(i,0) )
        result +="%9s     %6d     %6d     %6d     %6d     %6d   %7s\n" % (
                i, wl_info.get(i,0), prgs_info.get(i,0), ml_info.get(i,0), fl_info.get(i,0), al_info.get(i,0), percent)
    percentage = "%3.2f%%" % (prgs_total * 100.00 / wl_total)
    result +='%9s     %6d     %6d     %6d     %6d     %6d   %7s\n' %('-Total-',wl_total,prgs_total,ml_total,fl_total,al_total, percentage)
    result += '\n Used : ' + str(time.time() - start_time) + " seconds.\n"
    print(result)
    dumpInfo(result)


